@@ -34,7 +34,7 @@ sub load {
 
         $host = lc $host;
         return $mc->err("invalid host pattern: '$host'")
-            unless $host =~ /^[\w\-\_\.\*]+$/;
+            unless $host =~ /^[\w\-\_\.\*\;\:]+$/;
 
         $ss->{extra_config}->{_vhosts} ||= {};
         $ss->{extra_config}->{_vhosts}{$host} = $target;
@@ -83,9 +83,29 @@ sub vhost_selector {
     return $cb->_simple_response(404, "Not Found (no reqheaders)") unless $req;
 
     my $vhost = $req->header("Host");
-
+    my $uri = $req->request_uri;
     my $maps = $cb->{service}{extra_config}{_vhosts} ||= {};
 
+    # ability to ask for one host, but actually use another.  (for
+    # circumventing javascript/java/browser host restrictions when you
+    # actually control two domains).
+    if ($vhost && $uri =~ m!^/__using/([\w\.]+)(?:/\w+)(?:\?.*)?$!) {
+        my $alt_host = $1;
+
+        # update our request object's Host header, if we ended up switching them
+        # around with /__using/...
+        my $svc_name = $maps->{"$vhost;using:$alt_host"};
+        my $svc = $svc_name ? Perlbal->service($svc_name) : undef;
+        unless ($svc) {
+            $cb->_simple_response(404, "Vhost twiddling not configured for requested pair.");
+            return 1;
+        }
+
+        $req->header("Host", $alt_host);
+        $svc->adopt_base_client($cb);
+        return 1;
+    }
+
     # returns 1 if done with client, 0 if no action taken
     my $map_using = sub {
         my ($match_on, $force) = @_;
